      SUBROUTINE msgind_ges_wTQ(nsat,brt,psurf,theta,alat,
     &  amonth,pland,T_air,Q_air,temp,wvmr,ozmr,tsurf,tpw,toz,
     &  emisir,init)

C-----------------------------------------------------------------------
C !F77
C
C !DESCRIPTION:
C     SEVIRI regression retrieval of atmospheric profiles of
C     temperature, moisture, and ozone at 101 levels. In addition,
C     surface skin temperature, and IR surface
C     emissivities and reflectivities are retrieved.
C     algorithm developed by Jun Li, CIMSS/SSEC.
C
C !INPUT PARAMETERS:
C     nsat     METSAT satellite number 8,9,10,....
C     brt      Brightness temperatures (K) for all 12 SEVIRI band
C              Local array KUSE determines which bands are used.
C     psurf    Surface pressure (hPa)
C     theta    Local zenith angle (aka senzor zenith angle, degrees)
C     alat     Latitude (degrees, -90S to +90N)
C     amonth   Month of year (1-12)
C     pland    Percentage of land
C     T_air    suplementary temperature profile (101 levels) (K)
C     Q_air    suplementary moisture profile (101 levels)  (g/kg)
c     init     retrieval initialization
C
C !OUTPUT PARAMETERS:
C     temp     Retrieved profile of atmospheric temperature (K)
C     wvmr     Retrieved profile of water vapor mixing ratio (g/kg)
C     ozmr     Ozone mixing ratio profile (ppmv) 
C     tsurf    Surface skin temperature (K)
C     tpw      Total precipitable water vapor (cm)
C     toz      Total column ozone (DU)
C     emisir   8 IR emissivities at SEVIRI bands (bands 4 - 11)
C
C !REVISION HISTORY:
C
C !TEAM-UNIQUE HEADER:
C     Developed by the GOES/MODIS Groups, CIMSS/SSEC, UW-Madison.
C
C !END
C-----------------------------------------------------------------------

      IMPLICIT none

c ... General parameters
      INTEGER nl, nT, nQ, nemis, nall, leng
      PARAMETER (nl = 101)                     ! 101 layers
      PARAMETER (nT = 54)                      ! 54 layers between 100 and 1050 hPa
      PARAMETER (nQ = 35)                      ! 35 layers between 300 and 1050 hPa
      PARAMETER (nemis = 8)                    ! 8 IR emissivities
      PARAMETER (nall = 12)                    ! 12 SEVIRI bands
      PARAMETER (leng = 3 * nl + 1 + nemis)    ! T/W/O + tskin + emis

      INTEGER nxx, nx, ny
      PARAMETER (nxx = 6)                      ! 6 IR bands used for regression
      PARAMETER (nx = nT + nQ + 2*nxx + 4)     ! number of predictors
      PARAMETER (ny = leng)                    ! number of predictands

      INTEGER numang, numsat
      PARAMETER (numsat =  1)                  ! So far, only 1 satellite (MSG-1) available
      PARAMETER (numang = 81)                  ! 81 view angles

c ... Parameters for positioning records in the binary file
      INTEGER nx1, nrx
      PARAMETER (nx1 = nx + 1)
      PARAMETER (nrx = nx1 * numsat * numang)

c ... Input
      INTEGER nsat
      REAL*4  brt(*)
      REAL*4  psurf
      REAL*4  pland
      REAL*4  theta
      REAL*4  alat
      REAL*4  amonth
      REAL*4  t_air(*)
      REAL*4  q_air(*)
      INTEGER init

c ... Output
      REAL*4  temp(*)
      REAL*4  wvmr(*)
      REAL*4  ozmr(*)
      REAL*4  tsurf
      REAL*4  tpw
      REAL*4  toz
      REAL*4  emisir(*)

c ... Local scalars
      REAL*4 sum, mindiff, thisdiff, wmax
      INTEGER i, ii, ixx, j, l, lang
      INTEGER iuc

c ... Local arrays
      REAL*4 angp(numang), coef(nx, ny), coeftw(nrx, ny),
     &  xtw(nx), ytw(ny), ytwb(ny)
      REAL*4 rtv(leng), p(nl)
      INTEGER kuse(nall)
      REAL*4 ang0, ang1, delang
      CHARACTER*88 twcoeff

c ... External functions
      REAL satmix
      EXTERNAL satmix
      
c ... Save statement
      SAVE
      
c ... Data statements

c ... Band use flags (1=use band, 0=don't use band)
      DATA kuse/3*0,0,2*1,0,4*1,0/

c ... Angle boundry
      DATA ang0/0.0/
      DATA ang1/81.0/

c ... Regression coefficient file logic number     
      DATA iuc/60/

c ... Pressure profile at 101 levels (hPa)
      DATA p/0.005,.016,.038,.077,.137,.224,.345,.506,.714,
     &  .975,1.297,1.687,2.153,2.701,3.340,4.077,4.920,
     &  5.878,6.957,8.165,9.512,11.004,12.649,14.456,16.432,
     &  18.585,20.922,23.453,26.183,29.121,32.274,35.651,39.257,
     &  43.100,47.188,51.528,56.126,60.989,66.125,71.540,77.240,
     &  83.231,89.520,96.114,103.017,110.237,117.777,125.646,133.846,
     &  142.385,151.266,160.496,170.078,180.018,190.320,200.989,212.028,
     &  223.441,235.234,247.408,259.969,272.919,286.262,300.000,314.137,
     &  328.675,343.618,358.966,374.724,390.893,407.474,424.470,441.882,
     &  459.712,477.961,496.630,515.720,535.232,555.167,575.525,596.306,
     &  617.511,639.140,661.192,683.667,706.565,729.886,753.628,777.790,
     &  802.371,827.371,852.788,878.620,904.866,931.524,958.591,986.067,
     &  1013.948,1042.232,1070.917,1100.000/

c-----------------------------------------------------------------------
c     INITIALIZATION
c-----------------------------------------------------------------------

c ... Intialize the output array
      DO i = 1, leng
         rtv(i) = 0.0
      ENDDO
      
c ... Read view angle and regression coefficient files if required
      IF (init .EQ. 0) THEN

	 init = 1

c ... view angle assignment
      delang=(ang1-ang0)/FLOAT(numang)
      DO i = 1, numang
         angp(i) = ang0 + (i-1) * delang
      ENDDO

C -----------------------------------------------------------
c ... Open the regression coefficient file
C -----------------------------------------------------------
      twcoeff ='../ancillary_data/RC_msg1_9km_15min_6ch.bin'

      OPEN(iuc,FILE=twcoeff,RECL=ny,ACCESS='direct',STATUS='old')

c ... Read regression coefficients

      DO i = 1, nrx
         read (iuc, REC=i) ytw

c ...    Save the coefficients          
         DO j = 1, ny
            coeftw(i, j) = ytw(j)
         ENDDO
          
      ENDDO

      CLOSE(iuc)

      ENDIF

c-----------------------------------------------------------------------
c     RETRIEVAL
c-----------------------------------------------------------------------
c ... Get coefficient index for this view angle

      lang = 1

      mindiff = ABS(angp(1) - theta)

      DO i = 2, numang

         thisdiff = ABS(angp(i) - theta)

         IF (thisdiff .LT. mindiff) THEN
             mindiff = thisdiff
             lang = i
         ENDIF

      ENDDO

      IF (mindiff .GT. delang) THEN
         PRINT *,'mindiff, delang in msgind_ges_wTQ', mindiff, delang
         PRINT *,'Input view angle is not within the angle clas range'
         STOP 8888
      ENDIF

c ... Get coefficients for this view angle

      DO i = 1,nx

	    ii=(nsat-8)*numang*nx1+(lang-1)*nx1+i

         DO j = 1, ny
            coef(i, j) = coeftw(ii, j)
         ENDDO

      ENDDO

      ii=(nsat-8)*numang*nx1+lang*nx1

      DO j = 1, ny
         ytwb(j) = coeftw(ii, j)
      ENDDO

c ... Linear term of regression equation

      ixx = 0

      DO l = 1, nall
         IF (kuse(l) .NE. 0) THEN
             ixx = ixx + 1
             xtw(ixx) = brt(l)
         ENDIF
      ENDDO

c ... Non-linear term of regression equation

      DO l = 1, nall
         IF (kuse(l) .NE. 0) THEN
             ixx = ixx + 1
             xtw(ixx) = brt(l) * brt(l)/250.0
         ENDIF
      ENDDO

c ... Additional predictors

c ... Surface pressure
      ixx = ixx + 1
      xtw(ixx) = psurf

c ... Percentage of land
      ixx = ixx + 1
      xtw(ixx) = pland

c ... Latitude
      ixx = ixx + 1
      xtw(ixx) = alat

c ... Month
      ixx = ixx + 1
      xtw(ixx) = amonth

c ... Air temperature
	 DO i =1, nT
	    ixx = ixx + 1
	    xtw(ixx) = T_air(i + 44)   ! from 100 to 1050 hPa
	 ENDDO

c ... Air humidity
	 DO i =1, nQ
	    ixx = ixx + 1
	    xtw(ixx) = q_air(i + 63)   ! from 300 to 1050 hPa
	 ENDDO

c ... Evaluate regression equation
      DO i = 1, ny
      
        sum = 0.0

        DO j = 1, nx
          sum = sum + xtw(j) * coef(j, i)
        end DO

        rtv(i) = sum + ytwb(i)
        
c ...   Water vapor mixing ratios and ozone partial pressures
c ...   are retrieved as logarithms, so convert to actual values
        if (i .LE. 3 * nl .AND. i .GT. nl) rtv(i) = EXP(rtv(i))
        
c ...   TPW retrieved as logarithms
c        if(i .EQ. 3*nl+2) rtv(i) = EXP(rtv(i))

c ...   TOZ retrieved as logarithms
c        if(i .EQ. 3*nl+3) rtv(i) = EXP(rtv(i))

c ...   Limit minimum retrieved value for any PARAMETER to 0.001        
        if (rtv(i) .LE. 0.001) rtv(i) = 0.001
        
      ENDDO

c ... Contents of array RTV (314 elements) are as follows:
c ...   Elements   1-101 atmospheric temperatures (K)
c ...   Elements 102-202 water vapor mixing ratios (g/kg)
c ...   Elements 203-303 ozone partial pressures (ppmv)
c ...   Element  304     surface skin temperature (K)
c ...   Element  305     tpw
c ...   Element  306     toz
c ...   Elements 307-314 emissivities

c ... Extract retrieved profiles
      DO i = 1, nl
         temp(i) = rtv(i)
         wvmr(i) = rtv(i + nl)
         ozmr(i) = rtv(i + (nl * 2))
      ENDDO

c ... DO not allow retrieved water vapor mixing ratios greater than
c ... saturation mixing ratio
      DO i = 1, nl
         wmax = satmix(p(i), temp(i))
         wvmr(i) = min(wvmr(i), wmax) 
      ENDDO

c-----------------------------------------------------------------------
c     DERIVED PRODUCTS
c-----------------------------------------------------------------------

      tsurf = rtv(3*nl+1)

c      tpw = rtv(3*nl+2)

c      toz = rtv(3*nl+3)

      DO i = 1, nemis
c         emisir(i) = rtv(3*nl+3+i)
         emisir(i) = rtv(3*nl+1+i)
      ENDDO

      END SUBROUTINE msgind_ges_wTQ