	SUBROUTINE seviri_phy_profile_EOF(
c-------INPUT----------------
     &  tb,		!  background of temperature, 101 pressure levels (K)
     &  wb,		!  background of water vapor, 101 pressure levels (g/kg)
     &  tsurfb,	!  background of surface skin temperature (K)
     &  b_inv,		!  inverse of background error covariance matrix
     &  tg,		!  first guess of temperature, 101 pressure levels (K)
     &  wg,		!  first guess of moisture, 101 pressure levels (g/kg)
     &  o3g,		!  first guess of ozone, 101 pressure levels (ppmv) ! no change in iteration
     &  tsurfg,	!  first guess of skin temperature (K)
     &  emisg,		!  first guess of surface emissivities, 8 IR spectral bands, no change in iterations
     &  brto,		!  bias-adjusted BT (K), 12 channels, 0 to VIS/NIR bands
     &  banduse,	!  band use index
     &  radnos,	!  NeDR of SEVIRI, 12 spectral bands
     &  isfc,		!  1 to use surface obs, 0 not to use surface obs
     &  tsair,		!  surface obs of temperature (K)
     &  wsair,		!  surface obs of moisture (g/Kg)
     &  nsat,		!  Meteosat satellite index (8)
     &  theta,		!  local zenith angle (degree)
     &  psurf,		!  surface pressure (hPa)
     &  lsfc,		!  surface layer index in the 101-layer system
     &  initr,		!  initilization index: set initr=0 in the begaining of the main program
     &  IF_print,	!  1 to print out some iteration information
     &  ib_covar,	!  0 to use defaut background covariance matrix, 1 to use input covariance matrix

c-------OUTPUT-----------------
     &  tr,		!  physical retrieval of temperauture, 101 pressure levels (K)
     &  wr,		!  physical retrieval of moisture, 101 pressure levels (g/kg)
     &  tsurfr,	!  physical retrieval of skin temperature (K)
     &  IF_success,	!  0 means failure to physical retrieval, others mean successful retrieval
     &  residual0,	!  BT residual from the first guess at 12 spectral bands (K)
     &  residual1,	!  BT residual from the physical retrieval at 12 spectral bands (K)
     &  qc_flag)	!  quality flag in the physical retrieval

c--------------------------------------------------------------------
C 	Subtoutine for physical retrieval (101 pressure levels)
c--------------------------------------------------------------------

C !DESCRIPTION:

c--------------------------------------------------------------------
c       Quasi-Newton Iteration form:
c--------------------------------------------------------------------

c  x(n+1) = xb + [gamf*B^(-1) + K'(n)*E^(-1)*K(n)]^(-1)*K'(n)*E^(-1)*[(Yo-Yc(x(n))) + K(n)*(x(n)-xb)]

c	where:
c	   x 		is the parameters needed to be retrieved
c	   xb		is the background 
c	   x(0)		is the first guess for iteration
c	   B 		is the background error covariance matrix
c 	   K			is weighting function
c	   ^(-1) 		stands for inverse
c	   E			is the observation error covariance matrix
c	   Yo		is observation vector (e.g., BT)
c	   Yc	 	is calculation vector (e.g., BT)
c	   gamf		is the smoothing factor which helps the convergence

c--------------------------------------------------------------------

c--------------------------------------------------------------------
c       Transfer to profile EOF domain (or using basis functions)
c--------------------------------------------------------------------
c	Define: x(n+1)-xb = C * V
c	Define: Kc = K * V
c	Define: Bc^(-1) = V' * B^(-1) * V

C the iteration form in profile EOF domain

c  C(n+1) = [gamf*Bc^(-1) + Kc'(n)*E^(-1)*Kc(n)]^(-1)*Kc'(n)*E^(-1)*[(Yo-Yc(C(n))) + Kc(n)*C(n)]

c--------------------------------------------------------------------

c    This is the subroutine to DO physical iterations. The subroutine of 
c    RTVSOLX_GN_PC calculate the C(n+1) and x(n+1) (refer to above equations) during each
c    iteration. Here is the main flow chart:

C *   read in covariance matrices and eigen vectors IF needed, and re-map 
C * 	covariance matrices into eigen vector domain
C *			     ||
C *			     V
C *		covariance matrix of satellite observations
C *			     ||
C *			     V
C *		      iteration starts
C *			     ||
C *			     V
C *		calculate BT, K, convert K to eigenvector domain for Kc
C *			     ||
C *			     V
C *		     calculate and examine residuals
C *			||		  ||
C *			V		  V
C *	    very small residuals    very larger esiduals
C *			||		  ||
C *			V		  V
C *		    stop iteration	calculate x(i+1) (CALL RTVSOLX_GN_PC)
C *					  ||
C *					  V
C *		         CONTINUE iteration until max iteration or other
C *				termination conditions meet

C *	    	    exit from iteration
C *			     ||
C *			     V
C *	select best iteration based on averaged residuals, examine the
C *	retrieval quality based on averaged residuals
C
C !INPUT PARAMETERS:
C	tb         background of temperature, 101 pressure levels (K)
C	wb         background of water vapor, 101 pressure levels (g/kg)
C	tsurfb     background of surface skin temperature (K)
C	b_inv      background error covariance matrix
C	tg         first guess of temperature, 101 pressure levels (K)
C	wg         first guess of moisture, 101 pressure levels (g/kg)
C	o3g        first guess of ozone, 101 pressure levels (ppmv)	     
C	tsurfg     first guess of skin temperature (K)
C	emisg      first guess of surface emissivities, 8 IR spectral bands
C	brto       bias-adjusted BT (K), 12 spectral bands
C	banduse    SEVIRI band use index 
C	radnos     NEDR for SEVIRI, 12 spectral bands
C	isfc       1 to use surface obs, 0 not to use surface obs
C	tsair      surface obs of temperature (K)
C	wsair      surface obs of moisture (g/Kg)
C	nsat       meteosat satellite index (8)
C	theta      local zenith angle (degree)
C	psurf      surface pressure (hPa)
C	lsfc       level for surface pressure
C	initr      index to show whether the SEVIRI spectrum information is read in or not
C	IF_print   1 to print out some iteration details, 0 otherwise
C	ib_covar   0 to use default error covariance, 1 for input error covariance
C
C !OUTPUT PARAMETERS:
C	tr         physical retrieval of temperauture, 101 layers (K)
C	wr         physical retrieval of moisture, 101 layers (g/kg)
C	tsurfr     physical retrieval of skin temperature (K)
C	IF_success 0 means failure to physical retrieval, others mean successful retrieval
C	residual0  residual from the first guess (brto - brtc(x(0))
C	residual1  residual from the physical retrieval (brto - brtc(x(n))
C	qc_flag    quality flag in the physical retrieval
C
C !REVISION HISTORY:
C
C !TEAM-UNIQUE HEADER:
C     Developed by the GOES/SOUNDING Groups, CIMSS/SSEC, UW-Madison.
C
c--------------------------------------------------------------------
C *		Programer: Jun Li
C *		CIMSS/SSEC
C *		1225 West Dayton Street
C *		Madison, WI 53706
C *		E-mail: Jun.Li@ssec.wisc.edu
C *		Tel: (608)-262-3755
C *		Fax: (608)-262-5974
C * For METEOSAT-8 
C
C !END
C***********************************************************************

c------------------------------------
c   some constant parameters
c------------------------------------
	IMPLICIT none
	INTEGER*4 nl       		   ! number of layers used in the RT model
	INTEGER*4 leng     		   ! number of variables needed to be retrieved (T,W and tskin)
	INTEGER*4 ncseviri 		   ! number of channels of SEVIRI
	INTEGER*4 nnew     		   ! number of 'pseudo' channels, including the sfc obs (2)
	INTEGER*4 nw       		   ! number of eigenvector for moisture used
	INTEGER*4 nt       		   ! number of eigenvector for temperature used
	INTEGER*4 ntot     		   ! total number of eigenvectors used (T,W and tskin)
	INTEGER*4 max_iteration    ! maximum number of iteration
	INTEGER*4 nemis            ! number of IR surface emissivities

	PARAMETER (nl = 101)
	PARAMETER (leng = 2*nl + 1)
	PARAMETER (ncseviri = 12)
	PARAMETER (nnew = ncseviri + 2) 
	PARAMETER (nw = 3)
	PARAMETER (nt = 1)
	PARAMETER (ntot = nt + nw + 1)
	PARAMETER (max_iteration = 7)
	PARAMETER (nemis = 8)

c -------------------------------------
c  INPUT
c -------------------------------------

	REAL*4    tb(*)		   ! background of temperature, 101 pressure levels (K)
	REAL*4    wb(*)		   ! background of water vapor, 101 pressure levels (g/kg)
	REAL*4    tsurfb		   ! background of surface skin temperature (K)
	REAL*4    b_inv(leng,leng) ! background error covariance matrix
	REAL*4    tg(*)		   ! first guess of temperature, 101 pressure levels (K)
	REAL*4    wg(*)		   ! first guess of moisture, 101 pressure levels (g/kg)
	REAL*4    o3g(*)		   ! first guess of ozone, 101 pressure levels (ppmv)	     
	REAL*4    tsurfg		   ! first guess of skin temperature (K)
	REAL*4    emisg(*)		   ! first guess of surface emissivities, 8 IR spectral bands
	REAL*4    brto(*)		   ! bias-adjusted BT (K), 12 spectral bands
	INTEGER*4 banduse(*)	   ! SEVIRI band use index 
	REAL*4    radnos(*)		   ! NEDR for SEVIRI, 12 spectral bands
	INTEGER*4 isfc	     	   ! 1 to use surface obs, 0 not to use surface obs
	REAL*4    tsair		   ! surface obs of temperature (K)
	REAL*4    wsair		   ! surface obs of moisture (g/Kg)
	INTEGER*4 nsat			   ! meteosat satellite index (8)
	REAL*4    theta		   ! local zenith angle (degree)
	REAL*4    psurf		   ! surface pressure (hPa)
	INTEGER*4 lsfc	 		   ! level for surface pressure
	INTEGER*4 initr		   ! index to show whether the SEVIRI spectrum information is read in or not
	INTEGER*4 IF_print		   ! 1 to print out some iteration details, 0 otherwise
	INTEGER*4 ib_covar		   ! 0 to use default error covariance, 1 for input error covariance

c ----------------------------------------------------------------------------------------------------
c  OUTPUT
c ----------------------------------------------------------------------------------------------------

	REAL*4    tr(*)        	   ! physical retrieval of temperauture, 101 layers (K)
	REAL*4    wr(*)            ! physical retrieval of moisture, 101 layers (g/kg)
	REAL*4    tsurfr           ! physical retrieval of skin temperature (K)
	INTEGER*4 IF_success       ! 0 means failure to physical retrieval, others mean successful retrieval
	REAL*4    residual0(*)     ! residual from the first guess (brto - brtc(x(0))
	REAL*4    residual1(*)     ! residual from the physical retrieval (brto - brtc(x(n))
	INTEGER*4 qc_flag          ! quality flag in the physical retrieval

c --------------------------------------------------------------------------------------------------------
c  temperary variables
c --------------------------------------------------------------------------------------------------------

c Radiances and brightness temperatures

	REAL*4 wf(leng,nnew)       ! weighting function in normal coordinates
	REAL*4 tbbc(nnew)          ! calculaton vector BT (K)
	REAL*4 tbbo(nnew)          ! observation vector (K)
	REAL*4 del(nnew)           ! difference btwn calculated and observed BT
	REAL*4 deltb(nnew)         ! difference btwn calculated and observed BT, multipled by esr.
	REAL*4 dtbeof(nnew)        ! 

	REAL*8 coef0(ntot)         ! coefficents for eigenvectos during last iteration
	REAL*8 coef1(ntot)         ! coefficents for eigenvectos during current iteration
	REAL*4 p_eof(leng,ntot)    ! profile EOFs
	REAL*4 awf(nnew,ntot)      ! weigting function in eigenvector domain
	REAL*4 bc_inv(ntot,ntot)   ! V' * b_inv * V

c----------------------

	REAL*4 t(nl)               ! temperature profile (K), used during iterations
	REAL*4 w(nl)               ! moisture profile (g/kg), used during iterations
	REAL*4 o3(nl)              ! ozone profile (ppmv), used during iterations
	REAL*4 tsurf               ! skin temperature (K), used during iterations
	REAL*4 emis(nemis)         ! surface IR emissivity, used during iterations
	REAL*4 prtvb(leng)         ! background of T, W and tskin
	REAL*4 prtv0(leng)         ! first guess of T, W and tskin
	REAL*4 prtvf(leng)         ! T, W, and tskin during iteration
	REAL*4 prtvm(leng)         ! T, W, and tskin during iteration - during iteration

	REAL*4 twf(nl)             ! temperature weigting function
	REAL*4 qwf(nl)             ! moisture weigting function
	REAL*4 owf(nl)             ! ozone weigting function
	REAL*4 tswf                ! skin temperature weigting function
	
	REAL*4 taud(nl)            ! transmittance by dry air
	REAL*4 tauw(nl)            ! transmittance by water vapor
	REAL*4 tauo(nl)            ! transmittance by ozone
	REAL*4 tau(nl)             ! total transmittance
	
	REAL*4 esr_inv(nnew)       ! square root (diagonal of obervation error covariance matrix (in BT domain))

	REAL*4 dbdtb(nnew)         ! derivative of radiance with respect to BT, array
	
	INTEGER*4 i,j,l,lx,k,n     ! some loop indexes
	REAL*4    etx              ! for radiance noise
	REAL*4    dtbrms0          ! the rmse of TB on the previous step
	REAL*4    dtbrms1          ! the rmse of TB on the next step
	REAL*4    sum
	REAL*4    msgtbbw          ! calculate TB
	REAL*4    gamf
	REAL*4    dbdt_metsg       ! d(radiance)/d(TB)
	INTEGER*4 ksum
	INTEGER*4 ifail
	INTEGER*4 mpass


c Related to the core subroutine RTVSOLX
	REAL*8    xtx(ntot,ntot)
	REAL*8    xiv(ntot,ntot)
	REAL*4    xit(ntot,nnew)
	
	INTEGER*4 kuse(nnew)       ! actual spectral band use index
	REAL*4    pobs(nl)         ! pressure coordinates
	REAL*4    B_inv_def(leng)  ! diagonal of inverse of default error covariance matrix

C	-------------------------------------------------
C	Data block
C	-------------------------------------------------

	DATA pobs/0.005,.016,.038,.077,.137,.224,.345,.506,.714,
     &    .975,1.297,1.687,2.153,2.701,3.340,4.077,4.920,
     &    5.878,6.957,8.165,9.512,11.004,12.649,14.456,16.432,
     &    18.585,20.922,23.453,26.183,29.121,32.274,35.651,39.257,
     &    43.100,47.188,51.528,56.126,60.989,66.125,71.540,77.240,
     &    83.231,89.520,96.114,103.017,110.237,117.777,125.646,
     &    133.846,142.385,151.266,160.496,170.078,180.018,190.320,
     &    200.989,212.028,223.441,235.234,247.408,259.969,272.919,
     &    286.262,300.000,314.137,328.675,343.618,358.966,374.724,
     &    390.893,407.474,424.470,441.882,459.712,477.961,496.630,
     &    515.720,535.232,555.167,575.525,596.306,617.511,639.140,
     &    661.192,683.667,706.565,729.886,753.628,777.790,802.371,
     &    827.371,852.788,878.620,904.866,931.524,958.591,986.067,
     &	1013.948,1042.232,1070.917,1100.000/

C----------------------------------------------------------------------
C	Set default error covariance matrix (inverse) - diagonal
C----------------------------------------------------------------------
	DATA B_inv_def/
     &  7.7744, 7.7744, 7.7744, 7.7744, 9.2349, 9.3980, 9.2749, 9.1301,
     &  9.1185, 9.0020, 9.0370, 9.0601, 9.1490, 9.3696, 9.5414, 9.6677,
     &  9.7221, 9.6945, 9.6310, 9.6285, 9.5952, 9.6453, 9.7229, 9.7730,
     &  9.7916, 9.7856, 9.7958, 9.8372, 9.8512, 9.8327, 9.8347, 9.8411,
     &  9.8348, 9.8169, 9.7885, 9.6584, 7.1397, 4.5144, 6.7731, 4.4957,
     &  7.4116, 4.6707, 6.7786, 5.7846, 5.4101, 6.4795, 4.8964, 7.3098,
     &  3.9838, 7.4054, 5.6157, 7.4862, 8.4359, 8.5206, 7.7919, 6.6568,
     &  8.1520, 8.5816, 8.0056, 6.4807, 7.6577, 8.4212, 7.8350, 5.9710,
     &  7.9417, 8.1506, 6.6881, 7.1720, 8.0112, 6.5868, 6.7965, 6.2287,
     &  6.9712, 7.1573, 5.5930, 5.8740, 7.1216, 8.0647, 6.9693, 6.1195,
     &  7.6504, 4.8068, 7.2469, 5.0553, 6.7405, 5.0996, 7.6607, 7.6448,
     &  5.1600, 7.5689, 7.2462, 4.7383, 7.4790, 5.0101, 5.8505, 3.2193,
     &  8.3572, 7.8162, 7.8162, 7.8162, 7.8162,     ! temperature
     &  44*50.0000, 
     &  44.9352, 36.8988, 36.1197, 40.7510, 38.2116, 39.9849, 38.7400,
     &  43.5827, 44.6298, 44.3993, 43.5183, 42.8429, 45.3006, 45.5849,
     &  45.1237, 44.3367, 44.0138, 43.5509, 42.9513, 42.2988, 38.1880,
     &  36.8671, 30.0935, 36.3096, 35.9464, 26.4574, 29.7665, 22.1868,
     &  31.3954, 30.4947, 24.0882, 17.1368, 31.5022, 35.0005, 29.9078,
     &  28.6420, 31.8615, 19.2554, 30.3740, 22.3142, 24.8415, 23.5988,
     &  34.4641, 34.2764, 20.5715, 32.7862, 31.9941, 25.0712, 36.4194,
     &  30.8251, 36.3099, 37.6853, 43.7514, 41.7933, 41.7933, 41.7933,
     &  41.7933,                                     ! mixing ratio
     &  0.15/                                        ! skin temperature

c-------------------------------------------------------------------------
C	Common block
c------------------------------------------------------------------------------------
	COMMON/taudwo/taud,tauw,tauo    ! transmittance by dry air, water vapor and ozone

c------------------------------------------------------------------------------------
	
	SAVE
	
c---------------------------------------------------------------
c	Initialization
c---------------------------------------------------------------

	IF(initr.EQ.0) THEN	
	 
	initr = 1

c---------------------------------------------------------------
c    obtain 101 pressure level basis functions (eigen vectors)
c---------------------------------------------------------------

	CALL eof_T_W_profile101(p_eof,nl,leng,nt,nw,ntot)

c---------------------------------------------------------------
c    test EOFs, they should have been normalized !
c---------------------------------------------------------------

	IF(IF_print.EQ.1)THEN

	   WRITE(*,'('' P_EOF self_test '')')

	   DO i=1,ntot

	      WRITE(*,'(''Element number:'',i7)')n

	      sum=0.0

	      DO n=1,leng
	         sum=sum+p_eof(n,i)**2
	      ENDDO

	      IF(IF_print.EQ.1)THEN
	         WRITE(*,'(''SUM of p_eof is: '',f10.3)')sum
	      ENDIF

	   ENDDO

	ENDIF

C	---------------------------------------------------------------
C	Inverse of backgound error covariance matrix
C	---------------------------------------------------------------

	IF(ib_covar.EQ.0) THEN           ! use default

	   DO i=1,leng
	      DO j=1,leng
	         b_inv(i,j)=0.0
	      ENDDO
	   ENDDO

	   IF (banduse(7).EQ.0) THEN      ! 5 band mode

	     DO i=1,leng
	        IF(i.GT.101.AND.i.LE.195)THEN
 	          b_inv(i,i)=b_inv_def(i)*1.
	        ELSEIF(i.GT.196.AND.i.LE.200)THEN
	          b_inv(i,i)=b_inv_def(i)*10.
	        ELSEIF(i.GT.200.AND.i.LE.202)THEN
	          b_inv(i,i)=b_inv_def(i)*70.
	        ELSE
	          b_inv(i,i)=b_inv_def(i)*1.
	        ENDIF
	     ENDDO

	   ELSE                           ! 6 band mode

	     DO i=1,leng
	        IF(i.GT.101.AND.i.LE.200)THEN
 	          b_inv(i,i)=b_inv_def(i)*1.
	        ELSEIF(i.GT.200.AND.i.LE.202)THEN
	          b_inv(i,i)=b_inv_def(i)*80.
	        ELSE
	          b_inv(i,i)=b_inv_def(i)*1.
	        ENDIF
	     ENDDO

	   ENDIF

	ENDIF

C       -----------------------------------------------
C       Transfer B_inv to Bc_Inv (from mormal domain to EV domain)
C       -----------------------------------------------

         DO L=1,ntot
            DO i=1,ntot

            bc_inv(L,i)=0.0

            DO k=1,leng
               DO n=1,leng
               bc_inv(L,i)=bc_inv(L,i) + 
     &                    p_eof(n,L) * b_inv(n,k) * p_eof(k,i)
               ENDDO
            ENDDO

            ENDDO
         ENDDO

	ENDIF                ! END of Initialization

c---------------------------------------------------------------
c        ! measured SERVIR brightness temperatures, bais adjusted and band use index
c---------------------------------------------------------------

	DO k = 1, ncseviri
	   tbbo(k) = brto(k)
	   kuse(k) = banduse(k)
	ENDDO

c---------------------------------------------------------------
c       temperature and water vapor surface observatioins
c---------------------------------------------------------------

	IF(isfc.EQ.0) THEN  	! without surface obs

	   tbbo(nnew-1) = 273.0
	   tbbo(nnew) = ALOG(7.0)
	   kuse(nnew-1) = 0
	   kuse(nnew) = 0
	   esr_inv(nnew-1) = 0.0
	   esr_inv(nnew) = 0.0

	ELSE   			! with surface obs

	   tbbo(nnew-1) = tsair  
	   tbbo(nnew) = ALOG(wsair)
	   kuse(nnew-1) = 1
	   kuse(nnew) = 1
	   esr_inv(nnew-1) = 1.0/0.8    ! assume surface air temperature has 0.8 K observation error
	   esr_inv(nnew) = 1.0/0.1      ! assume surface moisture has 10% observation error

	ENDIF
	
c------------------------------------------------
c  	Set square root of observation error covariance matrix 
c    including sfc air temperature and water vapor
c	covariance matrix of observation (BT)
c---------------------------------------------------
	DO k=1,ncseviri

	   IF (kuse(k).NE.0) THEN

	      dbdtb(k)=dbdt_metsg(tbbo(k),k)      ! d(radiance)/d(TB)
	      etx=radnos(k)/dbdtb(k)              ! std of BT noise
	      esr_inv(k)=1.0/SQRT(etx*etx+0.0225) ! assume forward model has error of 0.15 K

	   ENDIF

	ENDDO

c-----------------------------------------------------------------
c  background of retrieval, from regression or forecast
c-----------------------------------------------------------------

	DO L=1,nl  !T, W
	   prtvb(L) = tb(L)
	   prtvb(L + nl) = wb(L)
	ENDDO

	prtvb(2*nl + 1) = tsurfb !surface temperature

c------------------------------------------------------------------
c  first guess (from regression, background or other source)
c------------------------------------------------------------------

	DO L=1,nl  !T, W
	   prtv0(L) = tg(L)
	   prtv0(L + nl) = wg(L)
	ENDDO

	prtv0(2*nl + 1) = tsurfg !surface temperature

C       -------------------------------------------------
C       PHYSICAL RETRIEVAL LOOP STARTS HERE
C       -------------------------------------------------

C       ------------------------------------
C       Begining  of solution iteration
C       ------------------------------------

        DO L=1,ntot
           coef0(L) = 0.0
           coef1(L) = 0.0
        ENDDO

        mpass = 0

        ifail = 0
        gamf  = 1.0
        qc_flag = 0

        DO L=1,leng
           prtvf(L) = prtv0(L)
        ENDDO

        dtbrms0 = 9999.0

C	---------------------------------------------------------
C	Atmospheric parameters for radiative transfer calculations
C	----------------------------------------------------------

	IF(IF_print.EQ.1) THEN
	   WRITE(*,'(''Pressure, T, Q, O3 for TBBC in physical'')')
	ENDIF

	DO L=1,nl

	   t(L) = tg(L)
	   w(L) = wg(L)
	   o3(L) = o3g(L)

	   IF(IF_print.EQ.1) THEN
	      WRITE(*,'(1x,4f10.3)')pobs(L),t(L),w(L),o3(L)
	   ENDIF

	ENDDO


	DO L=1,nemis
	   emis(L) = emisg(L)
	ENDDO

	IF(IF_print.EQ.1) THEN
	   WRITE(*,'(''EMIS in TBBC:'',4f10.3)')emis
	ENDIF

	tsurf = tsurfg

10000	CONTINUE

c	------------------------------------------------------------------
C	radiative transfer calculations ( tbbc )
c	------------------------------------------------------------------

c---------------------------------------------------------------------------
C	Loop over the channels
C---------------------------------------------------------------------------

	IF(IF_print.EQ.1) THEN
	   WRITE(*,'('' begin loop 2300 :'')')
	ENDIF

C       ------------------------------------------------------
C       weighting function (Jacobian:matrix dTBi/dXj) initialization
C       ------------------------------------------------------

	DO n = 1, nnew

        tbbc(L) = 0.

        DO L = 1, leng
           wf(L,n) = 0.0
        ENDDO

	ENDDO

c-------------------------------------------------------------------
c  calculate TBB based on retrieved profiles of t,w,o3 and tsurf
c-------------------------------------------------------------------

	DO 2300 n = 1, ncseviri

	   IF(kuse(n).NE.0) THEN 

	     CALL tranmetsg101(t,w,o3,theta,nsat,n,tau,2222)
c
C * DO RADIATIVE-TRANSFER CALCULATIONS

	     tbbc(n) = msgtbbw(pobs,t,tau,tsurf,emis(n-3),    ! bands 4 - 11 have emissivities
     &           psurf,lsfc,n)

C-------------------------------------------------
C   	Compute Jacobian maxtrix (w.f.)
C-------------------------------------------------

	     CALL jcb_seviri(n,emis(n-3),t,tsurf,tau,taud,      ! bands 4 - 11 have emissivities
     &         tauw,tauo,nl,lsfc,dbdtb(n),twf,qwf,owf,tswf)

	     DO l=1,lsfc
	        wf(l,n) = twf(l)          ! temperature
	        wf(l + nl, n) = qwf(l)    ! moisture
	     ENDDO

C	-----------------------------------------------
C	Surface skin temperature
C	-----------------------------------------------

          wf(leng,n) = tswf   !skin temperature

	   ENDIF              ! ENDIF for spectral band use index

2300	  CONTINUE      ! end of channels/observations loop

C	------------------------------------------------------
C	Assign surface "calculations"
C	------------------------------------------------------

	IF(kuse(nnew-1).NE.0) tbbc(nnew-1) = t(lsfc)   ! sfc air temperature

	IF(kuse(nnew).NE.0) tbbc(nnew)   = ALOG(w(lsfc))   ! sfc air moisture

C--------------------------------------------------------------------------------------------------
C	Assign "Jacobian" for surface T/q observations (treat surface obs as additional two bands)
C--------------------------------------------------------------------------------------------------

	IF(kuse(nnew-1).NE.0) wf(lsfc,nnew-1) = 1.0

	IF(kuse(nnew).NE.0)  wf(nl+lsfc,nnew) = 1.0

C       -------------------------------------------------------
C       WF ---------> AWF (transfer to profile EOF space
C       -------------------------------------------------------

	DO n=1,nnew
	   DO L=1,ntot

	      awf(n,L)=0.0

	      DO lx=1,leng
	         awf(n,L) = awf(n,L) + wf(lx,n) * p_eof(lx,L)
	      ENDDO

	   ENDDO
	ENDDO

c----------------------------------------------------------------
c	End of calculations for TBBC and Jacobian
c----------------------------------------------------------------

c       --------------------------------------------------------------
c       Calcualte DELTB and start inverse procedure
c       --------------------------------------------------------------
	
	IF(IF_print.EQ.1)THEN
	   WRITE(*,'('' Band TBBO, TBBC, DELTB:'')')
	ENDIF

	ksum = 0

	dtbrms1=0.0
	
	DO 2400 n = 1, nnew

	  IF(kuse(n).NE.0) THEN

C	---------------------------------------
C	Delta Yn
C	---------------------------------------

	   del(n) = tbbo(n) - tbbc(n)    

	   IF(IF_print.EQ.1)THEN
	      WRITE(*,'(1x,2i5,3f15.4)') n,kuse(n),tbbo(n),tbbc(n),del(n)
	   ENDIF

	   IF (n.le.(nnew-2)) THEN

	       dtbrms1 = dtbrms1 + del(n) * del(n) * FLOAT(kuse(n))
	       ksum = ksum + kuse(n)

	   ENDIF

C       ----------------------------------------------------
C       deltb - first part: E**(-1/2) * (delta Yn)
C       ----------------------------------------------------

	   deltb(n) = del(n) * esr_inv(n) * FLOAT(kuse(n))   ! first part - delta Yn

C       ----------------------------------------------------
C       dtbeof: E**(-1/2) * (Fn * Cn)
C       ----------------------------------------------------

	   DO L = 1, ntot
	      awf(n,L) = awf(n,L) * esr_inv(n) * FLOAT(kuse(n))
	   ENDDO

	   dtbeof(n)=0.0

	   DO L = 1, ntot
	      dtbeof(n) = dtbeof(n) + awf(n,L) * coef0(L) 
	   ENDDO

C       ----------------------------------------------------
C       deltb: E**(-1/2) * (delta Yn + Kn*Cn)
C       ----------------------------------------------------

	   deltb(n) = deltb(n) + dtbeof(n)

	   residual1(n) = del(n)

C       ----------------------------------------------------

	   IF(mpass.EQ.0.AND.ifail.EQ.0) THEN
	      residual0(n) = del(n)
	   ENDIF

	  ENDIF

2400	  CONTINUE    !  end of loops for spectral bands 

	dtbrms1 = SQRT(dtbrms1/FLOAT(ksum))

C       --------------------------------------------------------------
c       Compute solution matrix
C       --------------------------------------------------------------

C-----------------------------------------------------------------------------------
C * ((Fn*Fn + gamf*Bc**-1)**-1) * Fn (Note Fn here is multpled by E**(-1/2) already)
C * output is "XIT"
C--------------------- --------------------------------------------------------------

	   CALL RTVSOLX(awf,bc_inv,gamf,xtx,xiv,xit,ntot,nnew)

C----------------------------------------------------------------------------------------------
c  C(n+1) = [gamf*Bc^(-1) + Kc'(n)*E^(-1)*Kc(n)]^(-1)*Kc'(n)*E^(-1)*[(Yo-Yc(C(n))) + Kc(n)*C(n)]
C----------------------------------------------------------------------------------------------

	   DO L=1,ntot

           sum = 0.0

           DO n = 1, nnew
              sum = sum + xit(L,n) * deltb(n)
           ENDDO

           coef1(L) = sum                    !  C(n+1)

	   ENDDO

C       ----------------------------------------------------------------------------------------------
C       Save the previous iteration's result (in case iteration divergence, set to previous results)
C       ---------------------------------------------------------------------------------------------

	   DO j = 1, leng
           prtvm(j) = prtvf(j)
	   ENDDO

C       ----------------------------------------------------------------------------------------------
C       Profile retrieval (X(n+1) = C(n+1) * V)
C       ---------------------------------------------------------------------------------------------

	   DO j = 1, leng

           sum = 0.0

           DO L = 1, ntot
              sum = sum + p_eof(j,L) * coef1(L)
           ENDDO

           IF(j.GT.nl.AND.j.LE.(2*nl)) THEN
             prtvf(j) = prtvb(j) * EXP(sum)
           ELSE
             prtvf(j) = prtvb(j) + sum
           ENDIF

	   ENDDO

C       ---------------------------------------------------------------------------------------------

	   DO L=1,leng

	      IF(prtvf(L).GT.400.0.OR.prtvf(L).LT.0.0) THEN

              WRITE(*,'('' Bad physical retrieval happened'')')
              qc_flag = 5                    ! bad retrieval
              GOTO 26000

	      ENDIF

	   ENDDO

C       ------------------------------
C       Discrepancy principal
C       ------------------------------
	   IF(IF_print.EQ.1) THEN

	   WRITE(*,'(''Bands used, mpass, dtbrms0, dtbrms1'',2i5,2f15.7)')
     &         ksum, mpass, dtbrms0, dtbrms1

	   ENDIF

	   IF(dtbrms1.GT.dtbrms0) THEN

	      ifail = ifail + 1
	      gamf = gamf * 1.8

	      DO j = 1, leng
	         prtvf(j) = prtvm(j)
	      ENDDO

	      DO L = 1, ntot
	         coef1(L) = coef0(L)
	      ENDDO

	      dtbrms1 = dtbrms0

	   ELSE

	      mpass = mpass + 1
	      gamf = 0.80 * gamf
	      dtbrms0 = dtbrms1

	   ENDIF

C    --------------------------------------------------------------------------
C     Update profile for next iteration
C   --------------------------------------------------------------------------

	   DO L = 1, nl
           t(L)  = prtvf(L)
           w(L)  = prtvf(L + nl)
	      IF(w(L).LE.0.002) w(L) = 0.002
	   ENDDO

	   tsurf = prtvf(2*nl + 1)

	   IF(ifail.GE.3.OR.mpass.GT.max_iteration.OR.dtbrms1.LE.0.3) 
     &     GOTO 26000

C	-------------------------------------------------------------------------
C	For Profile EOF coeficient for next iteration
C	-------------------------------------------------------------------------

	   DO L = 1, ntot
	      coef0(L) = coef1(L)
	   ENDDO

	   dtbrms0 = dtbrms1

	   GOTO 10000              ! go for the next iteration

C       --------------------------------------------
C       EXIT retrieval iterations
C       --------------------------------------------

26000	IF(IF_print.EQ.1) THEN

	        WRITE(*,'('' EXIT AT MPASS=:'', i4)') mpass

	     ENDIF

C       --------------------------------------------
C       Final retrieval of profiles
C       --------------------------------------------

c *	 profile

	   DO L = 1, nl
	      tr(L) = t(L)
	      wr(L) = w(L)
        ENDDO

c *	surface skin temperature

	   tsurfr = tsurf

C       ---------------------------------------
c ****  Quality control 1 (non-covergence)
C       ---------------------------------------

	   IF(mpass.LE.1) THEN

	      qc_flag = 1

	      IF(IF_print.EQ.1) THEN

	         WRITE(*,'(''Non-converged iteration !'')')
	         WRITE(*,'(''Exit dtbrms1 !!!'',f15.7)') dtbrms1

	      ENDIF

	   ENDIF

C       ---------------------------------------
c ****  Quality control 2 (residual too large)
C       ---------------------------------------

	   IF(dtbrms1.GE.2.0) THEN

	      qc_flag = 2

	      IF(IF_print.EQ.1) THEN

	         WRITE(*,'(''Residual too large after RTVL !'')')
	         WRITE(*,'(''Exit dtbrms1 !!!'',f15.7)') dtbrms1

	      ENDIF

	   ENDIF

C       ---------------------------------------
c ****  Quality control 3 (non-completed converge)
C       ---------------------------------------

	   IF((ifail .GE. 3.0).AND.(dtbrms1.GE. 0.4)) THEN

	      qc_flag = 3

	      IF(IF_print.EQ.1) THEN

	         WRITE(*,'(''Converge stop after a few iterations !'')')
	         WRITE(*,'(''Exit dtbrms1 !!!'',f15.7)') dtbrms1

	      ENDIF

	   ENDIF

	   GOTO 2233

2222	 WRITE(*,'('' Problem in I/O of SEVIRI transmittance'')')

	   STOP 12345

2233	 CONTINUE 
	 
	 IF(IF_print.EQ.1) THEN

	    WRITE(*,'('' FINAL Retrievals !!!'')')

	    DO L = 1, nl
	       WRITE(*,'(4f8.3)')pobs(L),tr(L),wr(L)
	    ENDDO

	 ENDIF

	 RETURN
	 END
