	PROGRAM regcoef_msg_TQprof 
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C *             Programer: Jun Li,and Xin Jin
C *             CIMSS/SSEC
C *             1225 West Dayton Street
C *             Madison, WI 53706
C *             E-mail: JunL@ssec.wisc.edu
C *                     xjin@ssec.wisc.edu
C *
C DESCRIPTION:
C  - Generate the regression coefficients for the atmospheric
C    profile retrieval based on a globe-wide training database
C  - In this training process, the regression coefficients are
C    generated for different local zenith angles (LZA) which 
C    are limited between 0 and 80 Deg. For each LZA, all records
C    between -70 and +70 in latitude are used.
C  - In the training databse, the original surface emissivities
C    are provided at 8 SEVIRI IR bands as eigen coefficients. They
C    are re-constructed by combining with the eigen vector loaded
C    from 'ev_seviri_emis_be_h2l.bin'
C  - Three sets of SEVIRI spectral feathers for METEOSAT(8/9/10)
C    are prepared but only the NEDR array for METEOSET-8 is true
C    because only the in-flight NEDT of MSG-1/SEVIRI is available. 
C    If users can get the in-flight NEDT for other two SEVIRI 
C    instruments, they can calculate the corresponding NEDR and
C    updata the arrays 'radnos9' and 'radnos10'. So far, they are
C    filled with the same values as those in radnos8 to keep the 
C    code in a good shape.
C
C CORE VARIABLES:
C    x    : Forcast T/Q profile, TB, TB*TB, Psurf, Pland, Lat & Month, (Predictors; nx)
C    y    : T(p), W(p), O(p), Ts, emisir (predictands; ny)
C    nrec : Number of observations in sample
C    ee   : Error vector (squared) of predictors
C    xtx  : product of x-transpose and x 
C    xty  : product of x-transpose and y 
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

	USE iflport
	IMPLICIT none

	INTEGER*4 nl                    ! total Pressure layer number
	INTEGER*4 nT                    ! temperature layers between 100 and 1050 hPa
	INTEGER*4 nQ                    ! moisture layers between 300 and 1050 hPa
	INTEGER*4 leng                  ! record length of the training database
	INTEGER*4 ncseviri              ! number of SEVIRI IR band
	INTEGER*4 numsat                ! number of MSG satellite
	INTEGER*4 numang                ! number of LZA 
	INTEGER*4 nemis                 ! number of IR band for emissivity in the database

	PARAMETER (nl=101)
	PARAMETER (nT=54)
	PARAMETER (nQ=35)
	PARAMETER (leng=325)
	PARAMETER (ncseviri=8)
	PARAMETER (numsat=1)
 	PARAMETER (numang=81)
	PARAMETER (nemis=8)

C	-----------------------------------------------------------------
C	NONLINEAR REGRESSION 
C	-----------------------------------------------------------------
	INTEGER*4 nxx                   ! number of IR band used for regression
	INTEGER*4 nx                    ! number of predictors
	INTEGER*4 ny                    ! number of predictands
	INTEGER*4 nx1                   ! ancillary for data I/O
	PARAMETER (nxx=6)               ! band 5/6/8/9/10/11
	PARAMETER (nx=2*nxx+4+nT+nQ)
	PARAMETER (ny=3*nl+1+ncseviri)
	PARAMETER (nx1=nx+1)

C	-----------------------------------------------------------------
C	LZA-related variables
C	-----------------------------------------------------------------
	REAL*4 angcls(numang)
	REAL*4 ang0
	REAL*4 ang1
	REAL*4 delang
	PARAMETER (ang0=0.0, ang1=81.0)

C	-----------------------------------------------------------------
C	training database-related 
C	-----------------------------------------------------------------
	INTEGER*4 satuse(numsat)        ! satellite use index
	CHARACTER*80 trainingf          ! training database filename
	CHARACTER*80 twcoeff            ! file of storing regression coefficients
	REAL*4 brt(ncseviri)            ! simulated SEVIRI brighteness temperature
	REAL*4 t(nl)                    ! temperature profile
	REAL*4 w(nl)                    ! moisture profile
	REAL*4 o(nl)                    ! ozone profile
	REAL*4 prof(leng)               ! a record in the database
	REAL*4 t_air(nT)                ! temperature between 100 and 1050 hPa as predictors
	REAL*4 q_air(nQ)                ! moisture between 300 and 1050 hPa as predictors
	REAL*4 p(nl)                    ! Pressure ordinate

C	-----------------------------------------------------------------
C	VARIABLES IN TWO SUBROUTINES (XYVARM AND MREGRE)
C	-----------------------------------------------------------------
	REAL*8 xtx(nx,nx),xiv(nx,nx),xty(nx,ny),ybar(ny),yvar(ny)
	REAL*8 coef(nx,ny),ee(nx),xbar(nx),xdel(nx),ev(ny),se(ny),sy(ny)
	REAL*4 x(nx),y(ny),yb(ny)
	DATA ee/nx*0.01/                ! Assuming default error of each predictor is 1%

C	-----------------------------------------------------------------
C	VARIABLES FOR EMISSIVITY INTERPOLATION AND REGRESSION
C	-----------------------------------------------------------------
	INTEGER*4 kuse(ncseviri)        ! band use index
	CHARACTER*88 emisEVf            ! data file storing emissivity EV and mean
	REAL*8 emisEV(nemis,nemis)      ! emissivity EVs
	REAL*8 emisM(nemis)             ! Mean emissivity
	REAL*4 emisCOEF(nemis)          ! emissivity EV coefficient
	REAL*4 emis(nemis)              ! emisivity true

C	-----------------------------------------------------------------
C	VARIABLES FOR TRANSMITTANCE CALCULATION 
C	-----------------------------------------------------------------
	REAL*4 TAUT(NL),TAUD(NL),TAUW(NL),TAUO(NL)
	COMMON /taudwo/TAUD,TAUW,TAUO

C	-----------------------------------------------------------------
C	VARIABLES FOR I/O
C	-----------------------------------------------------------------
	INTEGER*4 innt,iub,iuc
	DATA innt/20/,iub/40/,iuc/41/

C	-------------------------------------------------------------
C	Local
C	-------------------------------------------------------------
	REAL*4 anoise                   ! function to generate random noise
	REAL*4 psurf                    ! surface pressure
	REAL*4 alat                     ! latitude
	REAL*4 alon                     ! longtitude
	REAL*4 amonth                   ! numerical month index
	REAL*4 pland                    ! land fraction index
	REAL*4 tsurf                    ! surface skin temperature
	REAL*4 msgtbbw                  ! function to simulate SEVIRI TB
	REAL*4 theta                    ! LZA
	REAL*4 dbdt_metsg               ! function to calculate (partial R)/(partial TB)
	REAL*4 xyx                      ! random noise
	INTEGER*4 lsfc, lsurface        ! Find the surface layer
	INTEGER*4 JX(ncseviri)          ! random number seeds
	INTEGER*4 nsat                  ! satellite ID (8 only)
	INTEGER*4 nrec                  ! total records in the database
	REAL*4 sum                      ! accumulator
	REAL*4 dbdtb                    ! partial R / partial TB
c	REAL*4 emis                     ! true emissivity at SEVIRI band from interpolation
	REAL*4 emistr                   ! true emissivity from outside
	REAL*4 etx                      ! for radiance noise calculation

	INTEGER*4 ixx,ixsav,nr,ns,k,nang,lenw,l,ii,i,j

C	-----------------------------------------------------------
C	MSG channel noise
C	-----------------------------------------------------------
	REAL*4 fac(ncseviri)            ! noise factor
	REAL*4 radnos(ncseviri)         ! NEDR
	REAL*4 radnos8(ncseviri)        ! MSG-1 NEDR; calcualted from in-flight NEDT
	REAL*4 radnos9(ncseviri)        ! MSG-2 NEDR; values same as that for MSG-1
	REAL*4 radnos10(ncseviri)       ! MSG-3 NEDR; values same as that for MSG-1

	DATA radnos8/0.004601, 0.009789, 0.022604, 0.094806,
     &              0.097523, 0.124708, 0.192272, 0.417790/
	DATA radnos9/0.004601, 0.009789, 0.022604, 0.094806,
     &              0.097523, 0.124708, 0.192272, 0.417790/
	DATA radnos10/0.004601,0.009789, 0.022604, 0.094806,
     &              0.097523, 0.124708, 0.192272, 0.417790/

C	------------------------------------------------------------
C	Channel used in regression
C	------------------------------------------------------------
	DATA kuse/1*0,2*1,1*0,4*1/

C    ---------------------------------------------------------
C	Satellite index
C    ---------------------------------------------------------
	DATA satuse/1*1/   ! Meteosat-8 only

C	---------------------------------------------------
C	Effective central wavenumbers (inverse centimeters)
C	---------------------------------------------------
	REAL*4 freq(ncseviri)
	REAL*4 freq8(ncseviri),freq9(ncseviri),freq10(ncseviri)
	DATA freq8/2566.0193, 1599.0001, 1361.3141, 1148.7864, 1034.2747,
     &           929.4522,  838.9161,  751.2728/
	DATA freq9/2568.3279, 1597.3705, 1359.5751, 1148.3387, 1035.2302,
     &           930.4792,  835.6805,  750.7132/
	DATA freq10/2565.8853,1594.5985, 1359.8208, 1147.8976, 1034.6736,
     &           928.7794,  837.9712,  749.6628/

	DATA p/0.005,.016,.038,.077,.137,.224,.345,.506,.714,
     &     .975, 1.297,   1.687,  2.153,  2.701,  3.340,  4.077,  4.920,
     &    5.878, 6.957,   8.165,  9.512, 11.004, 12.649, 14.456, 16.432,
     &   18.585, 20.922, 23.453, 26.183, 29.121, 32.274, 35.651, 39.257,
     &   43.100, 47.188, 51.528, 56.126, 60.989, 66.125, 71.540, 77.240,
     &   83.231, 89.520, 96.114,103.017,110.237,117.777,125.646,133.846,
     &  142.385,151.266,160.496,170.078,180.018,190.320,200.989,212.028,
     &  223.441,235.234,247.408,259.969,272.919,286.262,300.000,314.137,
     &  328.675,343.618,358.966,374.724,390.893,407.474,424.470,441.882,
     &  459.712,477.961,496.630,515.720,535.232,555.167,575.525,596.306,
     &  617.511,639.140,661.192,683.667,706.565,729.886,753.628,777.790,
     &  802.371,827.371,852.788,878.620,904.866,931.524,958.591,986.067,
     &  1013.948,1042.232,1070.917,1100.000/

C	---------------------------------------------------
C	T/W profile errors, obtained by comparing EC forecast and EC analysis profiles
C	---------------------------------------------------
	REAL*4 Tee(nT),Qee(nQ)   
	DATA Tee/3.9, 3.7, 3.5, 3.2, 3.0, 2.7, 2.5, 2.2, 1.9, 1.8, 1.8,
     &        1.8, 1.7, 1.7, 1.7, 1.9, 1.9, 1.9, 2.0, 2.1, 2.1, 2.1,
     &        2.2, 2.1, 2.1, 2.1, 2.1, 2.1, 2.0, 1.9, 1.9, 1.9, 1.9,
     &        1.9, 1.9, 1.9, 2.0, 2.1, 2.1, 2.2, 2.3, 2.4, 2.5, 2.5,
     &        2.6, 2.7, 2.7, 2.8, 2.9, 3.0, 3.1, 2.7, 2.2, 2.3/
	DATA Qee/34.0, 33.0, 34.1, 36.3, 38.4, 41.0, 43.5, 46.5, 49.5,
     &        53.6, 58.5, 57.3, 55.5, 52.9, 51.4, 50.6, 48.0, 46.2,
     &        46.1, 45.8, 44.0, 42.3, 38.9, 36.1, 33.8, 31.1, 27.4,
     &        22.2, 18.6, 16.0, 14.5, 13.6, 12.3, 10.3, 10.5/

C	----------------------------------------------
C	Content of training DATA
C	----------------------------------------------
c	1-101: temp
c	102-202: wv
c	203-303: oz
c	304: lat
c	305: lon
c	306: spres
c	307: tskin
c	308: wind speed
c	309: tpw
c	310: eco system
c	311: elev
c	312: land frac
c	313,314,315,316: year,month,day,hour
c	317: proftype
c	318-325: eigenvector coefficients of emissivity

C -----------------------------------------------------------
C	namelist
C -----------------------------------------------------------
	NAMELIST/regcoef_msg_nl/nrec,emistr,trainingf,emisEVf,
     & twcoeff,fac

C -----------------------------------------------------------
C	READ namelist
C -----------------------------------------------------------
	OPEN(2,FILE='regcoef_msg_TQprof.nl',STATUS='old')
	READ(2,regcoef_msg_nl)
	WRITE(*,regcoef_msg_nl)
	CLOSE(2)

C ------------------------------------------------------------
C	Initialization
C ------------------------------------------------------------
	DO k=1,ncseviri
	   jx(k) = 87347767+k*2
	ENDDO

C	---------------------------------------------------------------
C	Angle assignment
C	---------------------------------------------------------------
	delang = (ang1-ang0)/FLOAT(numang)
	DO k = 1, numang
 	   angcls(k)=ang0 + delang*(k-1)
	ENDDO

C	---------------------------------------------------------------
C	Open training file and regression coefficient file
C	---------------------------------------------------------------
	lenw=leng      ! linux
	OPEN(iub,FILE=trainingf,RECL=lenw,ACCESS='direct',STATUS='old',
     &      CONVERT='big_endian')
c
	lenw=ny        ! linux
	open(iuc,FILE=twcoeff,RECL=lenw,ACCESS='direct',STATUS='unknown')

C	-------------------------------------------------------------
C	Begin loop of MSG satellite
C	-------------------------------------------------------------
	DO 2000 nsat = 8, numsat+7

C	--------------------------------------------
C	Set noise and frequency
C	--------------------------------------------
	DO k=1,ncseviri
	   IF(nsat.EQ.8) THEN
              radnos(k) = radnos8(k)
              freq(k)   = freq8(k)
	   ENDIF
	   IF(nsat.EQ.9) THEN
	      radnos(k) = radnos9(k)
	      freq(k)   = freq9(k)
	   ENDIF
	   IF(nsat.EQ.10) THEN
	      radnos(k) = radnos10(k)
	      freq(k)   = freq10(k)
	   ENDIF
	ENDDO
       
C	------------------------------------
C	READ tranmittance coeff and initialization !!!
C	------------------------------------
	CALL pfcometsg101(nsat,700)
	GOTO 120

700	CONTINUE
	WRITE(*,'('' Problem in I/O of msg101pfc'')')
	STOP

800	CONTINUE
	WRITE(*,'('' Problem in I/O of tranmetsg101'')')
	STOP

120	CONTINUE

	IF(satuse(nsat-7).EQ.0) GOTO 2000

C	-------------------------------------------------------
C	open the emissivity EV file (real*8)
C	-------------------------------------------------------
	OPEN(2,FILE=emisEVf,RECL=nemis*2,ACCESS='direct',STATUS='old')

C	-------------------------------------------------------
C	load emissivity EVs
C	-------------------------------------------------------
	DO i=1,nemis

	   READ(2,REC=i) (emisM(j),j=1,nemis)

	   sum=0.0

	   DO j=1,nemis
	      emisEV(j,i)=emisM(j)
	      sum=sum+emisEV(j,i)*emisEV(j,i)
	   ENDDO

	   WRITE(*,'(''NO of EV, Sum'',i7,f10.3)')i,sum

	ENDDO

 	READ(2, REC=nemis+1) emisM

	CLOSE(2)

C	-----------------------------------------------
C	Angle classification
C	-----------------------------------------------
	DO 1000 nang = 1,numang

	   theta = angcls(nang)

	   ns = 0

	   CALL xyvarm(x,y,xdel,xbar,ybar,yvar,xtx,xty,nx,ny,ns,-1)

	   WRITE(*,'('' Begin to READ Training profile and
     $                 calculation of TB'')')

C	-----------------------------------------------------
C	Profile loop with training DATA
C	-----------------------------------------------------
	DO 1100 nr = 1,nrec

	   READ(iub,REC=nr) prof

	   alat   = prof(304)
	   alon   = prof(305)
	   psurf  = prof(306)
	   tsurf  = prof(307)
	   pland  = prof(312)
	   amonth = prof(314)

	   IF(ABS(alat).GT.70.0) GOTO 1100

C	--------------------------------------------
c	Find surface level in pressure profile
C	--------------------------------------------
	   lsfc = lsurface(nl,p,psurf,200.0,1080.0)

C	------------------------------------------
C	Calculate brightness temperatures for MSG
C	------------------------------------------
	   DO j=1,nl
	      t(j)=prof(j)
	      w(j)=prof(j+nl)*1000.0
	      o(j)=prof(j+2*nl)
	   ENDDO
c
C	------------------------------------------
C	Get emissivity EV coefficients
C	------------------------------------------
	   DO j=1,nemis
	      emisCOEF(j)=prof(317+j)
	   ENDDO

C	------------------------------------------
C	Get emissivity of this record
C	------------------------------------------
	   DO j=1,nemis

	      emis(j)=emisM(j)

	      DO L=1,nemis
	         emis(j)=emis(j) + emisCOEF(L) * emisEV(j,L)
	      ENDDO

c	  WRITE(*,'(''J, EMIS(J)'',i7,f10.3)')j, emis(j)

	   ENDDO
c
	   DO 160 k=1,ncseviri

	   IF(kuse(k).EQ.0) GOTO 160

c
	   CALL tranmetsg101(t,w,o,theta,nsat,K+3,taut,800)

C * DO RADIATIVE-TRANSFER CALCULATIONS
	   brt(K)=msgtbbw(p,t,taut,tsurf,emis(k),psurf,lsfc,k+3)

C	----------------------------------------
C	Calculate partial R / partial TB
C	----------------------------------------
	   dbdtb=dbdt_metsg(brt(k),k+3)

C	-----------------------------------------
C	Convert NeDR to NeDT @ BRT(K)
C	-----------------------------------------
	   etx=fac(k)*radnos(k)/dbdtb

C	---------------------------------------
C	Assume forward model error is 0.2K
C	---------------------------------------
	   etx = SQRT(etx*etx + 0.2*0.2) 
	   xyx = anoise(JX(k),etx,0.)
	   brt(k) = brt(k) + xyx 

160	   CONTINUE

C --------------------------------------------------
C	Add T/W profiles as additional predictors
C --------------------------------------------------
	   DO j=1,nT
	      t_air(j)=t(j+44)     ! 100 hPa < P < 1050 hPa
	   ENDDO

	   DO j=1,nQ
	      q_air(j)=w(j+63)     ! 300 hPa <= P < 1050 hPa
	   ENDDO

C ------------------------------------------
C	Set up predictands
C ------------------------------------------

c ... Add T/W/O3 profiles to predictand
	   DO i=1,nl

	      y(i)=t(i)

	      y(i+nl)=ALOG(w(i))
	      IF(w(i).LE.0.002) y(i+nl)=ALOG(0.002)

	      y(i+2*nl)=ALOG(o(i))
	      IF(o(i).LE.0.001) y(i+2*nl)=ALOG(0.001)

	   ENDDO

c ... Add tsurf to predictand
	   y(3*nl+1)=tsurf

c ... Add emissivity to predictand
	   DO L=1,ncseviri
	      y(3*nl+1+L)=emis(L)
	   ENDDO

C ------------------------------------------
C	Set up predictors
C ------------------------------------------

c------------------------------------------
c 	linear 
c------------------------------------------
	ixx=0

	DO L=1,ncseviri
	   IF(kuse(L).NE.0) THEN
	      ixx=ixx+1
	      x(ixx)=brt(L)
	   ENDIF
	ENDDO

c------------------------------------------
c 	Tbb**2 terms 
c------------------------------------------
	DO L=1,ncseviri
	   IF(kuse(L).NE.0) THEN
	      ixx=ixx+1
	      x(ixx)=brt(L)*brt(L)/250.0
	   ENDIF
	ENDDO
c
	ixsav=ixx

C --------------------------------------------
C	Surface Pressure
C --------------------------------------------
	ixx=ixx+1
	x(ixx)=psurf 

C --------------------------------------------
C	Percentage of land
C --------------------------------------------
	ixx=ixx+1
	x(ixx)=pland

c -------------------------------------------
C 	Latitude
c -------------------------------------------
	ixx=ixx+1
	x(ixx)=alat

C-------------------------------------------
C	Month
C-------------------------------------------
	ixx=ixx+1
	x(ixx)=amonth

C-------------------------------------------
C	air temperatures
C-------------------------------------------
	DO L=1,nT
	   ixx=ixx+1
	   x(ixx)=t_air(L)
	ENDDO

C-------------------------------------------
C	air moistures
C-------------------------------------------
	DO L=1,nQ
	   ixx=ixx+1
	   x(ixx)=ALOG(q_air(L))
	   IF (q_air(L).LE.0.002) x(ixx)=ALOG(0.002)
	ENDDO

	IF (ixx.NE.nx) PRINT *, 'ixx .ne. nx'

	ns=ns+1

	CALL xyvarm(x,y,xdel,xbar,ybar,yvar,xtx,xty,nx,ny,ns,0)
C	---------------------------------------------------

1100	CONTINUE

	CALL xyvarm(x,y,xdel,xbar,ybar,yvar,xtx,xty,nx,ny,ns,1)

C ------------------------------------------
C	set up error matrix
C ------------------------------------------
C * Surface pressure
	ee(ixsav+1)=10.0*10.0

C * Percentage of Land
	ee(ixsav+2)=0.01*0.01

C * Latitude
	ee(ixsav+3)=0.1*0.1

c * Month
	ee(ixsav+4)=0.0001

c * T profile
	DO L=1,nT
	   ee(ixsav+4+L)=Tee(L)*Tee(L)*1.0*1.0
	ENDDO

c * Q profile
	DO L=1,nQ
	   ee(ixsav+4+nT+L)=0.01*Qee(L)*Qee(L)*0.01
	ENDDO
	
C	WRITE(*,'('' SATELLIE NO & THETA:'',i6,f10.3)')nsat,theta

	CALL mregre(ee,xtx,xty,xtx,xiv,coef,yvar,ev,sy,se,nx,ny)

	PRINT *, ' theta & nang:', theta, nang

	IF (MOD(nang,10).EQ.1) THEN

	   WRITE(*,'(''      ybar     yvar      ev      sy       se'')')

	   DO j=1,ny
	      WRITE(*,'(1x,i5,2f10.3,3d12.3)') j,ybar(j),yvar(j),ev(j)
     + ,sy(j),se(j)
        ENDDO    

	ENDIF

	DO i=1,nx

	   DO j=1,ny
	      y(j)=SNGL(coef(i,j))
	   ENDDO

	   ii=(nsat-8)*numang*nx1+(nang-1)*nx1+i

	   WRITE(iuc,REC=ii) y

	ENDDO

	DO j=1,ny

	   sum=0.0

	   DO i=1,nx
	      sum=sum+SNGL(coef(i,j)*xbar(i))
	   ENDDO

	   yb(j)=ybar(j)-sum

	ENDDO	
	
	ii=(nsat-8)*numang*nx1+nang*nx1

	WRITE(*,*)numang,nx1,nx,ny,ii

	WRITE(iuc,REC=ii) yb

1000	CONTINUE
2000	CONTINUE

	GOTO 300
200	CONTINUE

	WRITE(*,'('' ERROR IN READING GOESRT !!'')')
	STOP 1111

300	CONTINUE

	CLOSE(iub)
	CLOSE(iuc)

	STOP
	END
